<?php

namespace Waggybytes\SimplyRemindMe\Library\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\Auth;
use Waggybytes\SimplyRemindMe\Library\Localisation\DateTimeHandler;
use Waggybytes\SimplyRemindMe\Library\Localisation\DateTimeHandlerFormatter;

class Event extends Model
{
    use HasFactory;

    const REPEAT_PERIOD_NONE = 'none';

    protected $casts = [
        'event_due_at' => 'datetime',
        'is_maintenance_completed' => 'boolean',
        'is_public' => 'boolean',
        'public_social_sharing' => 'boolean',
        'reminders_paused_at' => 'datetime'
    ];

    protected $fillable = [
        'user_id',
        'title',
        'event_due_at',
        'completed_at',
        'repeat_period',
        'event_icon',
        'parent_event_id',
        'is_maintenance_completed',
        'is_public',
        'public_url',
        'public_countdown_option',
        'public_description',
        'event_background',
        'public_social_sharing',
        'event_template_id',
        'notes',
        'event_version',
        'is_warning_reminders_enabled',
        'advanced_repeat',
        'warning_reminders'
    ];

    protected $hidden = [
        'warning_reminders'
    ];

    public static function findEventForCurrentUser($eventID)
    {
        /** @var Event $event */
        return Event::where([
            'user_id' => Auth::user()->id,
            'id' => intval($eventID)
        ])->first();
    }

    public static function validationRulesOnCreate()
    {
        // Reminders can be added as an array of children on the Event model, therefore add the singular Reminder rules
        // as array rules.
        // https://laravel.com/docs/5.3/validation#validating-arrays
        $reminderRules = [];
        foreach (Reminder::validationRulesOnCreate() as $key => $value)
        {
            $reminderRules['reminders.*.' . $key] = $value;
        }

        return array_merge(
            [
                'reminders' => 'required',
                'event_due_at' => 'required|max:19',
                'title' => 'required|max:255',
                'repeat_period' => 'required|max:20',
                'event_template_id' => 'sometimes|exists:event_templates,id'
            ],
            $reminderRules
        );
    }

    public static function validationRulesOnUpdate()
    {
        return [
            'event_due_at' => 'required|max:19',
            'title' => 'required|max:255',
            'repeat_period' => 'required|max:20'
        ];
    }

    public function getDueDateDiff(&$hasSpecificTime)
    {
        $eventDueAtTime = DateTimeHandler::using($this->event_due_at)
            ->convertTimezone()->toUser($this->user)
            ->format()->toCustom('Hi');

        $hasSpecificTime = intval($eventDueAtTime) > 0;

        $dueDate = !$hasSpecificTime
            ? $this->event_due_at->addDays(1)->addSeconds(-1) // No specific time, use 23:59:59
            : $this->event_due_at;

        $nowDate = DateTimeHandler::now()->current();

        return $nowDate->diffAsDateInterval($dueDate);
    }

    public function getDueDays()
    {
        $eventHasSpecificTime = false;
        $diff = $this->getDueDateDiff($eventHasSpecificTime);

        return $diff->invert
            ? 0
            : $diff->days;
    }

    public function getDueInText()
    {
        $eventHasSpecificTime = false;
        $diff = $this->getDueDateDiff($eventHasSpecificTime);

        if ($diff->invert)
        {
            return trans('reminders_module.event.show.due_passed');
        }
        elseif ($diff->days == 0 && $diff->h == 0)
        {
            return trans('reminders_module.event.show.due_now');
        }

        $numberOfDays = $diff->days;
        $numberOfHours = $diff->h;

        if ($numberOfDays == 0)
        {
            // Due today
            if (!$eventHasSpecificTime)
            {
                return trans('reminders_module.event.show.due_today');
            }
            else
            {
                return trans_choice('reminders_module.event.show.due_today_with_hours', $numberOfHours, ['number' => $numberOfHours]);
            }
        }
        else if ($numberOfDays == 1 && !$eventHasSpecificTime)
        {
            return trans('reminders_module.event.show.due_tomorrow');
        }

        if (!$eventHasSpecificTime)
        {
            return trans_choice('reminders_module.event.show.due_in_days', $numberOfDays, ['number' => $numberOfDays]);
        }
        else
        {
            $dayString = trans_choice('reminders_module.event.show.due_in_days', $numberOfDays, ['number' => $numberOfDays]);
            $hourString = trans_choice('reminders_module.event.show.due_in_hours', $numberOfHours, ['number' => $numberOfHours]);

            return implode(', ', [$dayString, $hourString]);
        }
    }

    public function reminderHistories()
    {
        return $this->hasManyThrough(ReminderHistory::class, Reminder::class);
    }

    public function reminderQueues()
    {
        return $this->hasManyThrough(ReminderQueue::class, Reminder::class);
    }

    public function reminders()
    {
        return $this->hasMany(Reminder::class);
    }

    public function toArray()
    {
        $data = parent::toArray();

        /** @var User $user */
        $user = Auth::user();

        $data['event_due_at'] = DateTimeHandler::using($this->event_due_at)
            ->convertTimezone()->toUser($user)
            ->format()->toCustom(DateTimeHandlerFormatter::API_DATE_FORMAT);

        // Advance reminders
        $data['reminders'] = [];
        foreach ($this->reminders as $reminder)
        {
            $data['reminders'][] = $reminder->toArray();
        }

        // Warning reminders
        if (!empty($this->warning_reminders))
        {
            $data['warning_reminders'] = json_decode($this->warning_reminders, true);
        }

        ksort($data);

        return $data;
    }

    public function user()
    {
        return $this->belongsTo(User::class);
    }
}