<?php

namespace Waggybytes\SimplyRemindMe\Library\Models;

use Carbon\Carbon;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Notifications\Notifiable;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Str;
use Waggybytes\SimplyRemindMe\Library\Localisation\DateTimeHandler;
use Waggybytes\SimplyRemindMe\Library\Localisation\DateTimeHandlerFormatter;

class Reminder extends Model
{
    use Notifiable, HasFactory;

    protected $casts = [
        'reminder_at' => 'datetime',
        'was_sent_while_paused' => 'boolean'
    ];

    protected $fillable = [
        'destination',
        'media_type_id',
        'reminder_at',
        'message',
        'voice_gender',
        'voice_language',
        'reminder_type',
        'is_warning_reminder',
        'relative_time',
        'was_added_in_past',
        'cancelled_at',
        'sent_at',
        'gave_up_at'
    ];

    protected $hidden = [
        'destination',
        'message',
        'relative_time',
        'reminder_type'
    ];

    /**
     * Formats the standard text for a reminder.
     * @param string $defaultTextKey Language resource key of the reminder text.
     * @param string $eventTitle Title of the event.
     * @param string $optionalMessage Optional message to include with the reminder.
     * @return string
     */
    public static function formatStandardReminderText(
        string $defaultTextKey,
        string $eventTitle,
        Carbon $eventDate,
        ?string $optionalMessage = null): string
    {
        $messageAddOn = trim($optionalMessage);
        if (!empty($messageAddOn))
        {
            if (!Str::endsWith($messageAddOn, ['.', '!']))
            {
                $messageAddOn .= '.';
            }

            if (!Str::endsWith($messageAddOn, ' '))
            {
                $messageAddOn .= ' ';
            }
        }

        return trans($defaultTextKey, [
            'event_title' => $eventTitle,
            'event_date' => $eventDate->format('d M Y'),
            'optional_text' => $messageAddOn
        ]);
    }

    public static function validationRulesOnCreate()
    {
        return [
            'media_type_id' => 'required|exists:media_types,id',
            'message' => 'sometimes',
            'reminder_at' => 'required|max:19'
        ];
    }

    public static function validationRulesOnUpdate()
    {
        // Update rules are no different from create
        return static::validationRulesOnCreate();
    }

    public static function validationRulesDestinationEmail()
    {
        return 'required|max:255|email';
    }

    public static function validationRulesDestinationSms()
    {
        return 'required|max:255|regex:/^\+[\d]{1,}\.[\d]{1,}$/';
    }

    public static function validationRulesDestinationVoice()
    {
        return 'required|max:255|regex:/^\+[\d]{1,}\.[\d]{1,}$/';
    }

    public function event()
    {
        return $this->belongsTo(Event::class);
    }

    public function mediaType()
    {
        return $this->belongsTo(MediaType::class);
    }

    public function queueItems()
    {
        return $this->hasMany(ReminderQueue::class);
    }

    public function getStatusInfo()
    {
        $status = ['code' => ''];

        if (is_null($this->sent_at) && is_null($this->gave_up_at) && is_null($this->cancelled_at))
        {
            $queueItem = $this->queueItems()->first();
            if (!is_null($queueItem))
            {
                $status['code'] = $queueItem->is_processing ? 'processing' : 'queued';
            }
            else
            {
                // Shouldn't ever get here as if there isn't a queue item, one of the above 3 fields should be set
                $status['code'] = 'cancelled';
            }
        }
        else if (!is_null($this->sent_at))
        {
            $status['code'] = $this->was_sent_while_paused ? 'sent_paused' : 'sent';
        }
        else if (!is_null($this->gave_up_at))
        {
            $status['code'] = 'failed';
        }
        else if (!is_null($this->cancelled_at))
        {
            $status['code'] = 'cancelled';
        }

        $status['description'] = trans(sprintf('global.reminder_status.%s', $status['code']));

        return $status;
    }

    /**
     * Get the notification routing information for the given driver.
     *
     * @param  string  $driver
     * @return mixed
     */
    public function routeNotificationFor($driver)
    {
        return $this->destination;
    }

    public function toArray()
    {
        $data = parent::toArray();
        $data['notes'] = $this->message;

        /** @var User $user */
        $user = Auth::user();

        $data['reminder_at'] = DateTimeHandler::using($this->reminder_at)
            ->convertTimezone()->toUser($user)
            ->format()->toCustom(DateTimeHandlerFormatter::API_DATE_FORMAT);

        // Reminder type
        switch ($this->reminder_type)
        {
            case 'A':
                $data['reminder_type'] = 'absolute';
                break;

            case 'R':
                $data['reminder_type'] = 'relative';
                break;
        }

        // Relative time
        if (!empty($this->relative_time) && strpos($this->relative_time, ':') !== false)
        {
            $relativeSplit = explode(':', $this->relative_time);
            $data['number_periods'] = intval($relativeSplit[0]);
            $data['period_type'] = $relativeSplit[1];
        }

        // Destination
        switch ($this->media_type_id)
        {
            case MediaType::email()->id:
                $data['email_address'] = $this->destination;
                break;

            case MediaType::sms()->id:
            case MediaType::voice()->id:
                $phoneSplit = explode('.', $this->destination);
                $data['country_code'] = $phoneSplit[0];
                $data['phone_number'] = $phoneSplit[1];
                break;
        }

        ksort($data);

        return $data;
    }
}