<?php

namespace Waggybytes\SimplyRemindMe\Library\Models;

use Illuminate\Contracts\Auth\MustVerifyEmail;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Notifications\Notifiable;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Support\Facades\DB;
use Laravel\Sanctum\HasApiTokens;

/**
 * @property string $email E-mail address of the customer.
 * @property string $first_name First name of the customer.
 * @property string $last_name Last name of the customer.
 * @property string $stripe_customer_id ID of the customer in Stripe.
 */
class User extends Authenticatable implements MustVerifyEmail
{
    use HasApiTokens, Notifiable, HasFactory;

    private $requiredSmsCount = -1;

    protected $casts = [
        'enable_sms_alert' => 'boolean',
        'is_beta_tester' => 'boolean',
        'is_profile_update_skipped' => 'boolean',
        'last_sms_credit_alert' => 'integer',
        'push_notification_all_reminders' => 'boolean',
        'sms_alert_level' => 'integer',
        'sms_credits' => 'integer'
    ];

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'first_name',
        'last_name',
        'email',
        'password',
        'is_activated',
        'activation_key',
        'activated_at',
        'sms_credits',
        'twitter_handle',
        'facebook_id',
        'avatar_url',
        'github_handle',
        'linkedin_id',
        'enable_sms_alert',
        'sms_alert_level',
        'last_sms_credit_alert',
        'mobile_number',
        'timezone_id',
        'last_sms_credit_alert_web',
        'is_profile_update_skipped',
        'default_reminder_at',
        'default_media_type_id'
    ];

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = [
        'password', 'remember_token', 'activation_key'
    ];

    public static function adminUsers()
    {
        return User::where([
            ['is_admin', true]
        ])->get();
    }

    public function announcements()
    {
        return $this->belongsToMany(Announcement::class, 'user_announcements')->withTimestamps();
    }

    public function avatar()
    {
        return $this->hasOne(UserAvatar::class);
    }

    public function avatarUrl($size = 80, $ifNotFound = 'mm')
    {
        if (!is_null($this->avatar_url))
        {
            return $this->avatar_url;
        }

        $emailHash = md5(strtolower(trim($this->email)));
        $gravatar = config('services')['gravatar'];

        return sprintf('%s%s.jpg?s=%d&d=%s', $gravatar['base_url'], $emailHash, $size, $ifNotFound);
    }

    public function configuration()
    {
        return $this->hasOne(UserConfiguration::class);
    }

    /**
     * @return UserAvatar
     */
    public function currentAvatarSettings()
    {
        $avatar = $this->avatar;
        if (is_null($avatar))
        {
            $avatar = UserAvatar::make($this);
        }

        return $avatar;
    }

    /**
     * @return UserConfiguration
     */
    public function currentConfiguration()
    {
        $config = $this->configuration;
        if (is_null($config))
        {
            $config = UserConfiguration::make($this);
        }

        return $config;
    }

    public function events()
    {
        return $this->hasMany(Event::class);
    }

    public function fullName()
    {
        return trim(sprintf('%s %s', $this->first_name, $this->last_name));
    }

    public function getDateTimeZone()
    {
        return new \DateTimeZone($this->timezone->identifier);
    }

    public function countSmsTopUpForQueue()
    {
        if ($this->requiredSmsCount == -1)
        {
            $mediaTypeSms = MediaType::sms();
            $mediaTypeVoice = MediaType::voice();

            $requiredSmsCost = DB::table('reminder_queue')
                ->join('reminders', 'reminder_queue.reminder_id', '=', 'reminders.id')
                ->join('events', 'reminders.event_id', '=', 'events.id')
                ->whereIn('reminders.media_type_id', [$mediaTypeSms->id, $mediaTypeVoice->id])
                ->where([
                    ['events.user_id', $this->id],
                    ['reminders.gave_up_at', null],
                    ['reminders.sent_at', null]
                ])
                ->sum('reminders.estimated_sms_credits');

            $this->requiredSmsCount = floatval($requiredSmsCost);
        }

        return $this->requiredSmsCount;
    }

    public function labs()
    {
        return $this->belongsToMany(Lab::class, 'user_labs');
    }

    public function needSmsTopUpForQueue()
    {
        return ($this->countSmsTopUpForQueue() > floatval($this->sms_credits));
    }

    public function pushNotificationSubscriptions()
    {
        return $this->hasMany(PushNotificationSubscription::class);
    }

    public function reminders()
    {
        return $this->hasManyThrough(Reminder::class, Event::class);
    }

    public function timezone()
    {
        return $this->belongsTo(Timezone::class);
    }
}